@@ -4,7 +4,7 @@
 (*  Copyright 2013 OCamlPro                                               *)
 (*                                                                        *)
 (*  All rights reserved.This file is distributed under the terms of the   *)
-(*  GNU Lesser General Public License version 3.0 with linking            *)
+(*  GNU Lesser General Public License version 2.1 with linking            *)
 (*  exception.                                                            *)
 (*                                                                        *)
 (*  TypeRex is distributed in the hope that it will be useful,            *)
@@ -58,7 +58,7 @@ let options =
   in
   let indent_empty =
     let doc = "Return indent for empty lines, too. Especially usefull \
-               with $(--numeric)." in
+               with $(b,--numeric)." in
     Arg.(value & flag & info ["indent-empty"] ~doc)
   in
   let lines =
@@ -144,6 +144,9 @@ let options =
     then `Error (false, "incompatible options used with --inplace")
     else if print_config then
       (let conf, synt,dlink = IndentConfig.local_default () in
+       let conf =
+         List.fold_left IndentConfig.update_from_string conf indent_config
+       in
        print_endline (IndentConfig.to_string ~sep:"\n" conf);
        if synt <> [] then
          Printf.printf "syntax = %s\n" (String.concat " " synt);
@@ -209,8 +212,8 @@ let info =
         option, or as a configuration definition in one of the following, \
         searched in order: a file named `.ocp-indent' in the current directory \
         or its parents (which allows for per-project indentation settings), \
-        the file `$HOME/.ocp/ocp-indent.conf', or the environment variable \
-        $OCP_INDENT_CONFIG."
+        the file `\\$HOME/.ocp/ocp-indent.conf', or the environment variable \
+        \\$OCP_INDENT_CONFIG."
   ] @
   IndentConfig.man
   @ [
