@@ -21,6 +21,8 @@ open Util
 
 module Node = struct
 
+  type extension_kind = ExtNode | Attr
+
   (* Node kind *)
   type kind =
     | KParen
@@ -41,6 +43,7 @@ module Node = struct
     | KArrow of kind
     | KColon
     | KType
+    | KConstraint
     | KException
     | KOpen
     | KInclude
@@ -69,8 +72,8 @@ module Node = struct
     | KWhen
     | KExternal
     | KCodeInComment
-    | KExtendedExpr of string list
-    | KExtendedItem of string list
+    | KExtendedExpr of string list * extension_kind
+    | KExtendedItem of string list * extension_kind
     | KAttrId of string list * bool
 
   (* Priority of open expression constructs (see below for operators) *)
@@ -88,6 +91,8 @@ module Node = struct
   (* Special operators that should break arrow indentation have this prio
      (eg monad operators, >>=) *)
   let prio_flatop = 59
+  let prio_colon = 35
+  let prio_arrow = 32
   let prio_semi = 5
 
   let rec follow = function
@@ -118,6 +123,7 @@ module Node = struct
     | KVerbatim _ -> "KVerbatim"
     | KUnknown -> "KUnknown"
     | KType -> "Ktype"
+    | KConstraint -> "KConstraint"
     | KException -> "KException"
     | KStruct -> "KStruct"
     | KSig -> "KSig"
@@ -136,10 +142,14 @@ module Node = struct
     | KWhen -> "KWhen"
     | KExternal -> "KExternal"
     | KCodeInComment -> "KCodeInComment"
-    | KExtendedExpr name ->
-        Printf.sprintf "KExtendedExpr(%s)" (String.concat "." (List.rev name))
-    | KExtendedItem name ->
-        Printf.sprintf "KExtendedItem(%s)" (String.concat "." (List.rev name))
+    | KExtendedExpr (name, kind) ->
+        Printf.sprintf "KExtendedExpr(%s,%s)"
+          (String.concat "." (List.rev name))
+          (match kind with ExtNode -> "node" | Attr -> "attr")
+    | KExtendedItem (name, kind) ->
+        Printf.sprintf "KExtendedItem(%s,%s)"
+          (String.concat "." (List.rev name))
+          (match kind with ExtNode -> "node" | Attr -> "attr")
     | KAttrId(name, dotted) ->
         Printf.sprintf "KAttrId(%s,%B)"
           (String.concat "." (List.rev name)) dotted
@@ -299,7 +309,7 @@ let unwind_while f path =
   | _ -> None
 
 let top_kind = function
-  | KStruct|KSig|KParen|KBegin|KObject|KExtendedItem _|KAttrId _ -> true
+  | KStruct|KSig|KParen|KBegin|KObject|KExtendedItem _|KAttrId _|KExtendedExpr _ -> true
   | _ -> false
 
 let stritem_kind = function
@@ -440,9 +450,9 @@ let op_prio_align_indent config =
   | OF -> 20,L,2
   | LESSMINUS | COLONEQUAL -> 20,L,config.i_base
   | COMMA -> 30,align,-2
-  | MINUSGREATER -> 32,L,0 (* is an operator only in types *)
-  | COLON -> 35,T,config.i_base
-  | COLONGREATER -> 35,L,config.i_base
+  | MINUSGREATER -> prio_arrow,L,0 (* is an operator only in types *)
+  | COLON -> prio_colon,T,config.i_base
+  | COLONGREATER -> prio_colon,L,config.i_base
   | OR | BARBAR -> 40,T,0
   | AMPERSAND | AMPERAMPER -> 50,T,0
   | (INFIXOP0 s | INFIXOP1 s | INFIXOP2 s | INFIXOP3 s | INFIXOP4 s)
@@ -477,16 +487,16 @@ let handle_dotted block tok =
     | _ -> false in
   let make_dotted_attr_id = function
     | { kind = KAttrId (names, _) } as node ::
-      ({ kind = (KExtendedItem [] | KExtendedExpr [])} :: _ as path) ->
+      ({ kind = (KExtendedItem ([],_) | KExtendedExpr ([],_))} :: _ as path) ->
         { node with kind = KAttrId (names, true) } :: path
     | _ -> assert false in
   let is_dotted_attr_id = function
-    | { kind = KExtendedExpr [] } :: _ -> true
-    | { kind = KExtendedItem [] } :: _ -> true
+    | { kind = KExtendedExpr ([],_) } :: _ -> true
+    | { kind = KExtendedItem ([],_) } :: _ -> true
     | { kind = KAttrId (_, dotted) } :: _ -> dotted
     | _ -> false in
   let make_attr_id name = function
-    | ({ kind = (KExtendedItem [] | KExtendedExpr []);
+    | ({ kind = (KExtendedItem ([],_) | KExtendedExpr ([],_));
          indent; pad; } :: _ as path) ->
           let indent =
             if starts_line then indent + pad
@@ -518,6 +528,11 @@ let handle_dotted block tok =
   else
     None
 
+let ext_kind = function
+  | LBRACKETPERCENT | LBRACKETPERCENTPERCENT -> ExtNode
+  | LBRACKETAT | LBRACKETATAT | LBRACKETATATAT -> Attr
+  | _ -> invalid_arg "ext_kind"
+
 (* Take a block, a token stream and a token.
    Return the new block stack. *)
 let rec update_path config block stream tok =
@@ -545,10 +560,10 @@ let rec update_path config block stream tok =
   (* Add a new child block *)
   let append kind pos ?(pad=config.i_base) = function
     | {kind = KAttrId (names, _)} ::
-      ({kind = KExtendedItem [] | KExtendedExpr [] } as n) :: path ->
+      ({kind = KExtendedItem ([],_) | KExtendedExpr ([],_) } as n) :: path ->
         let n = { n with kind = match n.kind with
-            | KExtendedItem [] -> KExtendedItem (List.rev names)
-            | KExtendedExpr [] -> KExtendedExpr (List.rev names)
+            | KExtendedItem ([],k) -> KExtendedItem (List.rev names,k)
+            | KExtendedExpr ([],k) -> KExtendedExpr (List.rev names,k)
             | _ -> assert false
           } in
         let path = {n with pad = config.i_ppx_stritem_ext } :: path in
@@ -699,8 +714,11 @@ let rec update_path config block stream tok =
         | KBegin -> path
         | KParen
           when if not config.i_align_ops then not starts_line else
-              match next_token stream with
-              | Some(SIG|STRUCT|OBJECT) -> true
+              match next_token_full stream with
+              | Some({token = SIG|STRUCT|OBJECT}, _) -> true
+              | Some({token = MODULE}, stream)
+                when next_token stream = Some STRUCT ->
+                  true
               | _ -> false
           -> path
         | _ ->
@@ -721,16 +739,19 @@ let rec update_path config block stream tok =
   in
   let make_infix tok path =
     let op_prio, align, indent = op_prio_align_indent config tok.token in
-    let in_record =
+    let in_delim_block () =
         match unwind_while (fun kind -> prio kind >= op_prio) path with
-        | Some ({ kind = KExpr _ } :: { kind = KBrace } :: _) -> true
+        | Some ({ kind = KExpr _; line } ::
+                { kind = (KBrace|KParen|KBracket|KBracketBar); line = bline } ::
+                _) ->
+            line = bline
         | _ -> false in
     (* special cases *)
     let indent =
       (* don't back-indent operators when alone on their line
          (except BAR because that would disrupt typing) *)
       if indent < 0 && tok.token <> BAR
-         && not (tok.token = SEMI && in_record)
+         && not (tok.token = SEMI && in_delim_block ())
          && next_offset tok stream = None
       then 0 else indent
     in
@@ -773,9 +794,14 @@ let rec update_path config block stream tok =
   | BEGIN       -> open_paren KBegin block.path
   | OBJECT      -> append KObject L block.path
   | VAL         -> append KVal L (unwind_top block.path)
-  | MATCH       ->
+  | MATCH | TRY ->
+      let k = match tok.token with
+        | MATCH -> KMatch
+        | TRY -> KTry
+        | _ -> assert false
+      in
       let p = fold_expr block.path in
-      if starts_line then append KMatch L p
+      if starts_line then append k L p
       else
         let enforce_strict =
           config.i_strict_with = Always
@@ -784,49 +810,26 @@ let rec update_path config block stream tok =
              | {kind=KBegin; indent; column} :: _ -> column = indent
              | _ -> false
         in
-        let p, pad =
-          if enforce_strict then
-            let p = reset_line_indent config current_line p in
-            reset_padding p, config.i_base
-          else p, Path.pad p + config.i_base
-        in
-        append KMatch L ~pad p
-  | TRY         ->
-      let p = fold_expr block.path in
-      if starts_line then append KTry L p
-      else
-        let enforce_strict =
-          config.i_strict_with = Always
-          || config.i_strict_with = Auto
-             && match p with
-             | {kind=KBegin; indent; column} :: _ -> column = indent
-             | _ -> false
-        in
-        let p, pad =
-          if enforce_strict then
-            let p = reset_line_indent config current_line p in
-            reset_padding p, config.i_base
-          else p, Path.pad p + config.i_base
-        in
-        append KTry L ~pad p
+        if enforce_strict then
+          let p = reset_line_indent config current_line p in
+          append k L (reset_padding p)
+        else
+          append k L ~pad:(Path.pad p + config.i_base) p
   | LPAREN -> open_paren KParen block.path
   | LBRACKET | LBRACKETGREATER | LBRACKETLESS ->
       open_paren KBracket block.path
   | LBRACKETPERCENT | LBRACKETAT ->
       let path = before_append_atom block.path in
-      append ~pad:4 (KExtendedExpr []) L path
+      append ~pad:4 (KExtendedExpr ([], ext_kind tok.token)) L path
   | LBRACKETATAT ->
       let path =
         (unwind (function KBody k | k -> top_kind k || stritem_kind k)
              block.path)
       in
-      let path = match path with
-        | {kind = KBody k | k} :: p -> if top_kind k then path else p
-        | [] -> []
-      in
-      append ~pad:4 (KExtendedItem []) L path
+      append ~pad:4 (KExtendedItem ([], ext_kind tok.token)) L
+        (reset_padding path)
   | LBRACKETPERCENTPERCENT | LBRACKETATATAT ->
-      append ~pad:4 (KExtendedItem []) L (unwind_top block.path)
+      append ~pad:4 (KExtendedItem ([], ext_kind tok.token)) L (unwind_top block.path)
   | LBRACKETBAR -> open_paren KBracketBar block.path
   | LBRACE | LBRACELESS ->
       open_paren KBrace block.path
@@ -861,11 +864,18 @@ let rec update_path config block stream tok =
         | _ -> assert false
       in
       let expr_start =
-        unwind (function KParen | KLet | KLetIn | KBody _ -> true | _ -> false)
+        unwind (function KParen | KBegin | KLet | KLetIn | KBody _ -> true
+                       | _ -> false)
           block.path
       in
       let indent = match expr_start with
-        | {kind=KParen}::{kind=KExpr prio; line; indent}::_
+        | {kind=KParen|KBegin}::{kind=KExpr prio}::
+          {kind=KBody KLet; line; indent; pad}::_
+          when prio = prio_apply && line = current_line ->
+            (* reset indent due to align_params for functor application within
+               [let module in] *)
+            indent + pad
+        | {kind=KParen|KBegin}::{kind=KExpr prio; line; indent}::_
           when prio = prio_apply && line = current_line ->
             indent
         | _ -> Path.indent block.path
@@ -876,7 +886,8 @@ let rec update_path config block stream tok =
   | WHEN ->
       append KWhen L ~pad:(config.i_base + if starts_line then 0 else 2)
         (unwind (function
-           | KWith(KTry|KMatch) | KBar(KTry|KMatch) | KFun -> true
+           | KWith(KTry|KMatch) | KBar(KTry|KMatch) | KFun | KExtendedExpr _ ->
+               true
            | _ -> false)
            block.path)
   | OPEN ->
@@ -918,7 +929,7 @@ let rec update_path config block stream tok =
           (function KType | KBody KType | KObject -> true | _ -> false)
           block.path
       in
-      append KLet L path
+      append KConstraint L path
 
   | AND ->
       let unwind_to = function
@@ -963,7 +974,8 @@ let rec update_path config block stream tok =
       (match last_token block with
        | Some LET ->
            append KUnknown L block.path (* let module *)
-       | Some COLON | Some EQUAL when next_token stream = Some TYPE ->
+       | Some (COLON|EQUAL|INCLUDE)
+         when next_token stream = Some TYPE ->
            append KUnknown L block.path (* : module type of *)
        | Some (WITH|AND) -> append KType L block.path
        | _ -> append KModule L (unwind_top block.path))
@@ -1080,11 +1092,16 @@ let rec update_path config block stream tok =
   | RBRACE | GREATERRBRACE -> close ((=) KBrace) block.path
 
   | RBRACKET ->
-      close
-        (function
-          | KBracket | KExtendedItem _ | KExtendedExpr _ -> true
-          | _ -> false)
-        block.path
+      let p =
+        unwind
+          (function
+            | KBracket | KExtendedItem _ | KExtendedExpr _ -> true
+            | _ -> false)
+          block.path
+      in
+      (match p with
+       | {kind=KExtendedItem (_, Attr)|KExtendedExpr (_, Attr)} :: p -> p
+       | p -> close (fun _ -> true) p)
 
   | GREATERRBRACKET -> close ((=) KBracket) block.path
 
@@ -1127,7 +1144,12 @@ let rec update_path config block stream tok =
         | {kind=KFun} :: ({kind=KExpr i} as e) :: path when i = prio_flatop ->
             (* eg '>>= fun x ->': indent like the top of the expression *)
             {e with kind = KExpr 0} :: path
-        | {kind=KFun; line } :: _
+        | {kind=KFun; line } :: {kind=KBody KLet; line=letline} :: _
+          when next_offset tok stream = None
+            && line = current_line && line <> letline
+          ->
+            append (KArrow KFun) L ~pad:0 (reset_line_indent config line path)
+        | {kind=KFun; line; _ } :: _
           when next_offset tok stream = None
             && line = current_line
           ->
@@ -1177,6 +1199,7 @@ let rec update_path config block stream tok =
         | KBar KType
         | KStruct | KSig | KObject
         | KAnd(KModule|KType|KLet|KLetIn)
+        | KConstraint
         | KExtendedItem _ | KExtendedExpr _ -> true
         | _ -> false
       in
@@ -1193,7 +1216,8 @@ let rec update_path config block stream tok =
               | _ -> replace (KBody KType) L ~pad:config.i_type path)
          | {kind=KBrace}::_ ->
              (match
-                unwind_while (fun kind -> prio kind > prio_semi) block.path
+                unwind_while (fun kind -> kind = KColon || prio kind > prio_semi)
+                  block.path
               with
               | Some ({kind=KExpr prio}::_) when prio = prio_semi + 1 ->
                   (* already after a field binding: this '=' must be
@@ -1208,6 +1232,8 @@ let rec update_path config block stream tok =
          | {kind=KAnd kind | kind} as h::p ->
              let indent = match next_token stream, kind with
                | Some (STRUCT|SIG), _ -> 0
+               | Some BAR, KType when config.i_strict_with = Always ->
+                   config.i_with
                | _, (KType | KBody KType) -> config.i_type
                | _ -> config.i_base
              in
@@ -1232,12 +1258,20 @@ let rec update_path config block stream tok =
   | COLON ->
       let path = unwind (function
           | KParen | KBegin | KBrace | KBracket | KBracketBar | KBody _
-          | KModule | KLet | KLetIn | KExternal | KVal
-          | KAnd(KModule|KLet|KLetIn) -> true
+          | KModule | KLet | KLetIn | KExternal | KVal | KColon
+          | KAnd(KModule|KLet|KLetIn) | KBar KType -> true
           | _ -> false)
           block.path
       in
       (match path with
+       | {kind = KBody(KVal|KType|KExternal) | KColon} :: _ ->
+           (match unwind_while (fun kind -> prio kind > prio_arrow) block.path
+            with
+            | Some path ->
+                extend (KExpr prio_colon)
+                  (if config.i_align_params = Never then L else T)
+                  path
+            | None -> make_infix tok block.path)
        | {kind = KModule|KLet|KLetIn|KExternal
          | KAnd(KModule|KLet|KLetIn|KExternal)} :: _ ->
            append KColon L path
@@ -1259,6 +1293,8 @@ let rec update_path config block stream tok =
               when i = prio_max && j = prio_apply -> (* "mutable" *)
                 extend KColon L p
             | _ -> make_infix tok block.path)
+       | {kind = KBar KType} :: _ ->
+           make_infix {tok with token = OF} block.path
        | _ -> make_infix tok block.path)
 
   | SEMI ->
@@ -1284,16 +1320,23 @@ let rec update_path config block stream tok =
       append KExternal L (unwind_top block.path)
 
   | DOT ->
-      let last_expr =
-        unwind_while (function KExpr _ -> true | _ -> false) block.path
-      in
-      (match last_expr with
+      (match block.path with
+       | {kind = KArrow KMatch} :: _ -> append expr_atom L block.path
+       | _ ->
+       let last_expr =
+         unwind_while (function KExpr _ -> true | _ -> false) block.path
+       in
+       match last_expr with
        | Some ({kind=KExpr _} :: {kind=KType} :: ({kind=KColon} :: _ as p)) ->
            (* let f: type t. t -> t = ... *)
            p
-       | Some ({kind=KExpr 200} :: ({kind=KColon} :: {kind=KLet} :: _ as p)) ->
+       | Some ({kind=KExpr 200} ::
+               ({kind=KColon} :: {kind=KLet|KLetIn} :: _ as p))->
            (* method m : 'x 'y . ... =   (KLet is actually "method") *)
-           p
+           (* let m : 'x 'y . ... =  (in) *)
+           (match last_token block with
+            | Some (UIDENT _) -> make_infix tok block.path
+            | _ -> p)
        | Some ({kind=KExpr i} :: ({kind=KBrace|KWith KBrace} as h :: p))
          when (i = prio_max || i = prio_dot) && next_offset tok stream = None ->
            (* special case: distributive { Module. field; field } *)
@@ -1345,6 +1388,17 @@ let rec update_path config block stream tok =
           | _ -> false)
           block.path
       with
+      | Some ( (* (opt)labels in types *)
+        {kind = KExpr 32 (* prio_arrow *)} ::
+        ({kind = KBody(KVal|KType|KExternal) | KColon} :: _) |
+        ({kind = KBody(KVal|KType|KExternal) | KColon} :: _)
+      ) ->
+          (* this is for the case [?foo:], parsed as OPTLABEL, but make sure we
+             are consistent with [foo:] or [? foo:], which are parsed as 2 or 3
+             tokens *)
+          extend (KExpr prio_colon)
+            (if config.i_align_params = Never then L else T)
+            (append expr_atom L block.path)
       | Some ({kind=KExpr _}::_) | None ->
           (* considered as infix, but forcing function application *)
           make_infix tok (fold_expr block.path)
@@ -1382,7 +1436,7 @@ let rec update_path config block stream tok =
   | FLOAT _ | CHAR _ | STRING _
   | TRUE | FALSE | NATIVEINT _
   | UNDERSCORE | TILDE | QUESTION
-  | QUOTE | QUOTATION ->
+  | QUOTE | QUOTATION _ ->
       atom block.path
 
   | PREFIXOP _ | BANG | QUESTIONQUESTION ->
